# Copyright 2021 Le Filament
# License AGPL-3.0 or later (https://www.gnu.org/licenses/agpl).

from odoo import fields, models


class ScopBordereauDeadlineWizard(models.TransientModel):
    _name = "scop.bordereau.deadline.wizard"
    _description = "Modification des dates d'échéance"

    # ------------------------------------------------------
    # Default functions
    # ------------------------------------------------------
    def _default_bordereau_id(self):
        bordereau_id = self.env["scop.bordereau"].browse(
            self.env.context.get("active_id")
        )
        return bordereau_id.id

    def _default_invoice_ids(self):
        bordereau_id = self.env["scop.bordereau"].browse(
            self.env.context.get("active_id")
        )
        invoice_ids = bordereau_id.invoice_ids.filtered(
            lambda i: i.payment_state in ["not_paid", "partial"]
        )
        default_invoices = invoice_ids.mapped(lambda i: (0, 0, {"invoice_id": i.id}))
        return default_invoices

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    bordereau_id = fields.Many2one(
        comodel_name="scop.bordereau",
        string="Bordereau",
        readonly=True,
        default=_default_bordereau_id,
    )
    partner_id = fields.Many2one(
        comodel_name="res.partner",
        string="Adhérent",
        related="bordereau_id.partner_id",
    )
    invoice_ids = fields.One2many(
        comodel_name="scop.bordereau.deadline.invoice.wizard",
        inverse_name="wizard_id",
        string="Appels de cotisation",
        default=_default_invoice_ids,
    )

    # ------------------------------------------------------
    # Constrains
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Override ORM
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Action
    # ------------------------------------------------------
    def change_deadline(self):
        self.ensure_one()
        self.invoice_ids.change_deadline()
        return {"type": "ir.actions.act_window_close"}


class ScopBordereauDeadlineInvoiceWizard(models.TransientModel):
    _name = "scop.bordereau.deadline.invoice.wizard"
    _description = "Appels de cotisation"

    wizard_id = fields.Many2one(
        comodel_name="scop.bordereau.deadline.wizard",
        string="Wizard",
        required=True,
        ondelete="cascade",
    )
    invoice_id = fields.Many2one(
        comodel_name="account.move",
        string="Appel de cotisation",
        required=True,
        readonly=True,
        ondelete="cascade",
    )
    old_deadline = fields.Date("Date d'échéance", related="invoice_id.invoice_date_due")
    quarter = fields.Selection(related="invoice_id.cotiz_quarter")
    type_contribution_id = fields.Many2one(
        comodel_name="scop.contribution.type",
        string="Type de cotisation",
        related="invoice_id.type_contribution_id",
    )
    new_deadline = fields.Date(
        "Nouvelle date d'échéance",
    )

    # ------------------------------------------------------
    # Action
    # ------------------------------------------------------
    def change_deadline(self):
        for line in self:
            if line.new_deadline:
                line.invoice_id.invoice_date_due = line.new_deadline
                aml_ids = line.invoice_id.line_ids.filtered(
                    lambda l: l.date_maturity == line.old_deadline
                )
                aml_ids.update({"date_maturity": line.new_deadline})
